# Selectricity: Voting Machinery for the Masses
# Copyright (C) 2007, 2008 Benjamin Mako Hill <mako@atdot.cc>
# Copyright (C) 2007 Massachusetts Institute of Technology
#
# This program is free software. Please see the COPYING file for
# details.

class Vote < ActiveRecord::Base
  # relationships to other classes
  belongs_to :voter
  has_many :rankings
  has_one :token
  
  # callbacks
  after_update :save_rankings
  before_destroy :destroy_rankings
  
  def to_s
    votes.join("")
  end

  def each
    self.votes.each {|vote| yield vote}
  end

  def votes
    unless @votes
      if rankings.empty?
        @votes = Array.new
      else
        @votes = self.rankings.sort.collect { |ranking| ranking.candidate.id }
      end
    end

    @votes
  end

  def votes=(array)
    @votes = array
  end

  def save_rankings
    self.votes # i need to initalize this before destroying rankings
               # or else the ranks themselves show up as nil

    destroy_rankings
    self.votes.each_with_index do |candidate_id, index| 
      ranking = Ranking.new
      ranking.rank = index
      ranking.candidate =  Candidate.find(candidate_id)
      self.rankings << ranking
    end
  end
  
  def destroy
    self.destroy_rankings
    super
  end

  def destroy_rankings 
    rankings.each { |ranking| ranking.destroy }
  end

  def confirm!
    self.confirmed = 1
    self.time = Time.now
    self.save
    
    unless self.voter.election.quickvote?
      token.destroy and token.reload if token
      self.token = Token.new
      self.save
    end
  end

  def confirm?
    confirmed == 1
  end
  
  def votestring
    # create a mapping of candidates ids and the relative order of the
    # candidates as they appear when sorted alphabetically
    cand_relnums = {}
    self.voter.election.candidates.sort.each_with_index do |c, i|
      cand_relnums[c.id] = i + 1
    end

    # assemble the votestring
    self.votes.collect {|v| (cand_relnums[v] + 64).chr}.join("")
  end

  # the following subroutine is used for quickvotes, but need for elections now
  # too. It creates a vote with the candidates listed in order of preference 
  # based on alphabetical order. Meant to be manipulated and then confirmed
  def set_defaults!  
    self.votes = self.voter.election.candidates.sort_by { rand }.collect {|c| c.id }
    self.save
  end
         
end
